@@ -19,6 +19,7 @@ from __future__ import absolute_import
 import copy
 import json
 import os
+import time
 import warnings
 import webbrowser
 
@@ -46,6 +47,11 @@ DEFAULT_PLOT_OPTIONS = {
     'sharing': files.FILE_CONTENT[files.CONFIG_FILE]['sharing']
 }
 
+SHARING_ERROR_MSG = (
+    "Whoops, sharing can only be set to either 'public', 'private', or "
+    "'secret'."
+)
+
 # test file permissions and make sure nothing is corrupted
 tools.ensure_local_plotly_files()
 
@@ -103,7 +109,7 @@ def _plot_option_logic(plot_options_from_call_signature):
 def iplot(figure_or_data, **plot_options):
     """Create a unique url for this plot in Plotly and open in IPython.
 
-    plot_options keyword agruments:
+    plot_options keyword arguments:
     filename (string) -- the name that will be associated with this figure
     fileopt ('new' | 'overwrite' | 'extend' | 'append')
         - 'new': create a new, unique url for this plot
@@ -161,7 +167,7 @@ def iplot(figure_or_data, **plot_options):
 def plot(figure_or_data, validate=True, **plot_options):
     """Create a unique url for this plot in Plotly and optionally open url.
 
-    plot_options keyword agruments:
+    plot_options keyword arguments:
     filename (string) -- the name that will be associated with this figure
     fileopt ('new' | 'overwrite' | 'extend' | 'append') -- 'new' creates a
         'new': create a new, unique url for this plot
@@ -248,7 +254,7 @@ def iplot_mpl(fig, resize=True, strip_style=False, update=None,
     2. makes a request to Plotly to save this figure in your account
     3. displays the image in your IPython output cell
 
-    Positional agruments:
+    Positional arguments:
     fig -- a figure object from matplotlib
 
     Keyword arguments:
@@ -281,7 +287,7 @@ def plot_mpl(fig, resize=True, strip_style=False, update=None, **plot_options):
     2. makes a request to Plotly to save this figure in your account
     3. opens your figure in a browser tab OR returns the unique figure url
 
-    Positional agruments:
+    Positional arguments:
     fig -- a figure object from matplotlib
 
     Keyword arguments:
@@ -1295,20 +1301,35 @@ def parse_grid_id_args(grid, grid_url):
             return grid.id
 
 
-def add_share_key_to_url(plot_url):
+def add_share_key_to_url(plot_url, attempt=0):
     """
-    Update plot's url to include the secret key
+    Check that share key is enabled and update url to include the secret key
 
     """
     urlsplit = six.moves.urllib.parse.urlparse(plot_url)
     username = urlsplit.path.split('/')[1].split('~')[1]
     idlocal = urlsplit.path.split('/')[2]
     fid = '{}:{}'.format(username, idlocal)
-
     body = {'share_key_enabled': True, 'world_readable': False}
     response = v2.files.update(fid, body)
 
-    return plot_url + '?share_key=' + response.json()['share_key']
+    # Sometimes a share key is added, but access is still denied.
+    # Check that share_key_enabled is set to true and
+    # retry if this is not the case
+    # https://github.com/plotly/streambed/issues/4089
+    time.sleep(4)
+    share_key_enabled = v2.files.retrieve(fid).json()['share_key_enabled']
+    if not share_key_enabled:
+        attempt += 1
+        if attempt == 50:
+            raise exceptions.PlotlyError(
+                "The sharekey could not be enabled at this time so the graph "
+                "is saved as private. Try again to save as 'secret' later."
+            )
+        add_share_key_to_url(plot_url, attempt)
+
+    url_share_key = plot_url + '?share_key=' + response.json()['share_key']
+    return url_share_key
 
 
 def _send_to_plotly(figure, **plot_options):
@@ -1504,6 +1525,81 @@ class dashboard_ops:
         return [str(dboard['filename']) for dboard in dashboards]
 
 
+class presentation_ops:
+    """
+    Interface to Plotly's Spectacle-Presentations API.
+    """
+    @classmethod
+    def upload(cls, presentation, filename, sharing='public', auto_open=True):
+        """
+        Function for uploading presentations to Plotly.
+
+        :param (dict) presentation: the JSON presentation to be uploaded. Use
+            plotly.presentation_objs.Presentation to create presentations
+            from a Markdown-like string.
+        :param (str) filename: the name of the presentation to be saved in
+            your Plotly account. Will overwrite a presentation of the same
+            name if it already exists in your files.
+        :param (str) sharing: can be set to either 'public', 'private'
+            or 'secret'. If 'public', your presentation will be viewable by
+            all other users. If 'private' only you can see your presentation.
+            If it is set to 'secret', the url will be returned with a string
+            of random characters appended to the url which is called a
+            sharekey. The point of a sharekey is that it makes the url very
+            hard to guess, but anyone with the url can view the presentation.
+        :param (bool) auto_open: automatically opens the presentation in the
+            browser.
+
+        See the documentation online for examples.
+        """
+        if sharing == 'public':
+            world_readable = True
+        elif sharing in ['private', 'secret']:
+            world_readable = False
+        else:
+            raise exceptions.PlotlyError(
+                SHARING_ERROR_MSG
+            )
+        data = {
+            'content': json.dumps(presentation),
+            'filename': filename,
+            'world_readable': world_readable
+        }
+
+        # lookup if pre-existing filename already exists
+        try:
+            lookup_res = v2.files.lookup(filename)
+            lookup_res.raise_for_status()
+            matching_file = json.loads(lookup_res.content)
+
+            if matching_file['filetype'] != 'spectacle_presentation':
+                raise exceptions.PlotlyError(
+                    "'{filename}' is already a {filetype} in your account. "
+                    "You can't overwrite a file that is not a spectacle_"
+                    "presentation. Please pick another filename.".format(
+                        filename=filename,
+                        filetype=matching_file['filetype']
+                    )
+                )
+            else:
+                old_fid = matching_file['fid']
+                res = v2.spectacle_presentations.update(old_fid, data)
+
+        except exceptions.PlotlyRequestError:
+            res = v2.spectacle_presentations.create(data)
+        res.raise_for_status()
+
+        url = res.json()['web_url']
+
+        if sharing == 'secret':
+            url = add_share_key_to_url(url)
+
+        if auto_open:
+            webbrowser.open_new(res.json()['web_url'])
+
+        return url
+
+
 def create_animations(figure, filename=None, sharing='public', auto_open=True):
     """
     BETA function that creates plots with animations via `frames`.
@@ -1696,8 +1792,7 @@ def create_animations(figure, filename=None, sharing='public', auto_open=True):
         body['share_key_enabled'] = True
     else:
         raise exceptions.PlotlyError(
-            "Whoops, sharing can only be set to either 'public', 'private', "
-            "or 'secret'."
+            SHARING_ERROR_MSG
         )
 
     response = v2.plots.create(body)
